#!/usr/bin/perl -w

# Copyright (C) 2006 Stphane Levant <sun@tuxfamily.org>

# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with GNU Emacs; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

# TODO:
# add support for MxN grid : add subsize_col and subsize_row
# add algorithms like (1,2) (1,2) or (1,2) (3,2) (2,3) in same line/col/bloc

my $VERSION = 0.1;

use strict;
use Pod::Usage;
use Getopt::Long;
Getopt::Long::Configure ("bundling");

# To avoid the error "Deep recursion on subroutine" which occured when
# a recursive function is called more than 100 times.
local $^W = 0;

# Options
my ($version,$man,$help);
my ($allsol, $info, $debug) = (0, 0, 0);
my $file;

# The program name
my $prog = $0;
$prog =~ s:/.+/::;

# The sudoku grid
my @grid;
# The size of the grid
my $size;
my $subsize;
# If letters are used instead of numbers
my $letter;

# The solutions (array of dim 3)
# Some bad grid can have more than one solution
my @solutions = ();

# Some informations
my @try_number = (0);



### Treat arguments and constructs grid

# Print an error message and exit
sub error {
    my ($message) = @_;
    print STDERR "$prog: $message\n";
    exit 2;
}

# Print version
sub version {
    print "$prog $VERSION

This is free software; see the source for copying conditions.  There is NO
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
";
    exit 0;    
}

# Treat options
sub TreatOptions {
    GetOptions
	(
	 'all|a'		=> \$allsol,
	 'debug|d'		=> \$debug,
	 'file|f=s'		=> \$file,
	 'info|i'		=> \$info,
	 'help|h|?'		=> \$help,
	 'man|m'		=> \$man,
	 'version|v'		=> \$version,
	 ) ||  pod2usage(2);

    pod2usage(1) if $help;
    pod2usage(-verbose => 2) if $man;
    version if ($version);
    if (@ARGV) {
	if (defined $file) {
	    error "Can't give both sudoku grid in argument and with a file";
	}
	&constructGrid(@ARGV);
    } else {
	if (defined $file) {
	    &readFile("$file");
	} else {
	    pod2usage(1);
	}
    }
}

sub verifySize {
    if ($size < 4) {
	error "size must be >= 4: $size"
    }
    if ((int sqrt($size))**2 != $size) {
	error "size must be a square: $size";
    }
}

sub readFile {
    my ($file) = @_;
    open(FILE, $file) || error "can't open file \"$file\"";
    if ($debug) {
	print "Read grid from file $file\n";
    }
    my $i = 0;
    my @a = ();
    while (<FILE>) {
	chop;
	if ($i == 0 && length($_) <= 2) {
	    if ($debug) {
		print "Found grid size (ignored): $_\n";
	    }
	} else {
	    push @a, $_;
	}
    }
    close(FILE);
    &constructGrid (@a);
}

# Constructs the grid from an array (see help)
sub constructGrid {
    my @t = @_;
    $size = scalar @t;
    &verifySize();
    $subsize = sqrt($size);
    $letter = -1;
    my $comma = -1;
    for (@t) {
	# Type : empty line sepated by comma
	if ($_ =~ /^([,\|._ \`\@]*)+$/ && $_ =~ /,\|/) {
	    $comma = 1;
	# Type : empty line
	} elsif ($_ =~ /^[._ \`\@]+/) {
	    $comma = 0;
	# Type : numbers sepated by comma
	} elsif ($_ =~ /^([0-9._ ,\|]*[,\|][0-9._ ,\|]*)+$/) {
	    if ($comma != -1 && $comma != 1
		|| $letter != -1 && $letter != 0 || $_ =~ /[a-zA-Z]/) {
		error "lines haven't the same format: $_";
	    }
	    $letter = 0;
	    $comma = 1;
	# Type : letters sepated by comma
	} elsif ($_ =~ /^([a-zA-Z._ \`\@,\|]*[,\|][a-zA-Z._ \`\@,\|]*)+/) {
	    if ($comma != -1 && $comma != 1
		|| $letter != -1 && $letter != 1 || $_ =~ /[0-9]/) {
		error "lines haven't the same format: $_";
	    }
	    $letter = 1;
	    $comma = 1;
	    if (length($_) > $size * 2 - 1) {
		error "must be only one character between comas: $_\n";
	    }
	# Type : attached letters
	} elsif ($_ =~ /^[a-zA-Z ._\`\@]+/) {
	    if ($comma != -1 && $comma != 0
		|| $letter != -1 && $letter != 1 || $_ =~ /[0-9]/) {
		error "lines haven't the same format: $_";
	    }
	    $letter = 1;
	    $comma = 0;
	    if ($size != length($_)) {
		error "line don't have $size elements: $_";
	    }
	# Type : attached numbers
	} elsif ($_ =~ /^[0-9 ._]+/) {
	    if ($comma != -1 && $comma != 0
		|| $letter != -1 && $letter != 0) {
		error "lines haven't the same format: $_";
	    }
	    $letter = 0;
	    $comma = 0;
	    if ($size != length($_)) {
		error "line don't have $size elements: $_";
	    }
	} else {
	    error "Bad format for this line: $_";
	}
    }
    my $i = 0;
    for (@t) {
	if ($comma == 0) {
	    for (my $j = 0; $j < length($_); $j++) {
		my $char = substr($_, $j, 1);
		if ($char =~ "[0 \`\@._]") {
		    $grid[$i][$j] = 0;
		} else {
		    if ($letter) {
			if (ord($char) > 96) {
			    $grid[$i][$j] = ord($char) - 96;
			} else {
			    $grid[$i][$j] = ord($char) - 64;
			}
		    } else {
			$grid[$i][$j] = $char;
		    }
		}
	    }
	} else {
	    my @a;
	    if ($_ =~ /,$/) {
		$_ .= " ";
	    }
	    if ($_ =~ /,/) {
		@a = split(",");
	    } else {
		@a = split("\\|");
	    }
	    if (@a != $size) {
		error "line don't have $size elements: $_";
	    }
	    for (my $j = 0; $j < length($_); $j++) {
		my $char = $a[$j];
		if ($char =~ /[\`0 \@]/ || $char eq "") {
		    $grid[$i][$j] = 0;
		} else {
		    if ($letter) {
			if (ord($char) >= 96) {
			    $grid[$i][$j] = ord($char) - 96;
			} else {
			    $grid[$i][$j] = ord($char) - 64;
			}
		    } else {
			$grid[$i][$j] = $char;
		    }
		}
	    }
	}
	$i++;
    }
    if ($info) {
	my $empty = getEmptyNumber();
	print "Original grid of ${size}x${size} with $empty empty cases:\n";
	printGrid();
	print "\n";
    }
}



### Util functions

# Calculate the number of empty case
sub getEmptyNumber {
    my $empty = 0;
    for (my $i = 0; $i < $size; $i++) {
	for (my $j = 0; $j < $size; $j++) {
	    $empty++ if ($grid[$i][$j] == 0);
	}
    }
    return $empty;
}

# Return the number or letter to display
sub sprintfCase {
    my ($num) = @_;
    if ($letter) {
	return "." if ($num == 0);
	return chr(96 + $num);
    } else {
	if ($size >= 10) {
	    return " ." if ($num == 0);
	    return sprintf("%3i", $num);
	} else {
	    return "." if ($num == 0);
	    return $num;
	}
    }
}

# Add the current grid to the solutions
sub addSolution {
    my $nbSol = @solutions;
    for (my $i = 0; $i < $size; $i++) {
	for (my $j = 0; $j < $size; $j++) {
	    $solutions[$nbSol][$i][$j] = $grid[$i][$j];
	}
    }    
}

# Prints the solutions
sub printSolutions {
    for (my $k = 0; $k < @solutions; $k++) {
	if (@solutions > 1) {
	    print "solution n" . ($k + 1) . "\n";
	}
	for (my $i = 0; $i < $size; $i++) {
	    for (my $j = 0; $j < $size; $j++) {
		print &sprintfCase($solutions[$k][$i][$j]);
	    }
	    print "\n";
	}
 	print "\n" if ($k != @solutions - 1);
    }
}

sub printGrid {
    for (my $i = 0; $i < $size; $i++) {
	for (my $j = 0; $j < $size; $j++) {
	    print &sprintfCase($grid[$i][$j]);
	}
	print "\n";
    }
}

# Verify if the solution is ok
sub verifySolution {
    for (my $i = 0; $i < $size; $i++) {
	for (my $j = 0; $j < $size; $j++) {
	    $grid[$i][$j] = $solutions[0][$i][$j];
	}
    }
    for (my $i = 0; $i < $size; $i++) {
	for (my $j = 0; $j < $size; $j++) {
	    my $number = $grid[$i][$j];
	    $grid[$i][$j] = 0;
	    my @a = &getPossibilies($i, $j);
	    $grid[$i][$j] = $number;
	    if (@a != 1 || $a[0] != $number) {
		return 0;
	    }
	}
    }
    return 1;
}


### Sudoku algorithm

sub getBlocIndice {
    my ($i, $j) = @_;
    return int ($i / $subsize) * $subsize + int ($j / $subsize);
}

# If the array contains one not-null objet, return the position of this object
# If the array contains more than one not-null objects, returns -2
# Returns -1 if the array contains all zeros : if this happens here,
# the sudoku has no solution.
sub getPosNumber {
    my (@ok) = @_;
    my $pos = -1;
    for (my $i = 0; $i < $size; $i++) {
	if ($ok[$i] != 0) {
	    if ($pos == -1) {
		$pos = $i;
	    } else {
		$pos = -2;
	    }
	}
    }
    return $pos;
}

# Try to place the number :
# Look for each line/column/bloc if the number can be place here
# Return the number of numbers placed or -1 if the grid has no solution
# NB: the code should be shorter with the function getPossibilies but
# this algorithm is the one a lot a people used manually.
# Furthermore, it should be easier to do other optimizations with
# an array of all possibilities for each case.
# Perhaps it will done in next version
sub placeNumber {
    my ($number) = @_;
    my $ret = 0;

    # Find all line/number/blocs where number is
    my @line = (1..$size);
    my @column = (1..$size);
    my @bloc = (1..$size);
    for (my $i = 0; $i < $size; $i++) {
	for (my $j = 0; $j < $size; $j++) {
	    if ($grid[$i][$j] == $number) {
		$line[$i] = 0;
		$column[$j] = 0;
		$bloc[&getBlocIndice($i, $j)] = 0;
	    }
	}
    }

    # Try to place number in all line
    for (my $i = 0; $i < $size; $i++) {
	if (!$line[$i]) {
	    next;
	}
	my @ok = (1..$size);
	for (my $j = 0; $j < $size; $j++) {
	    $ok[$j] = 0 if (!$column[$j] || $grid[$i][$j]);
	}
	for (my $k = 0; $k < $subsize; $k++) {
	    if (!$bloc[int ($i / $subsize) * $subsize + $k]) {
		for (my $x = 0; $x < $subsize; $x++) {
		    $ok[$k * $subsize + $x] = 0;
		}
	    }
	}
	my $pos = &getPosNumber(@ok);
	if ($pos == -1) {
	    if ($info) {
		print "Grid after Sudoku algorithm:\n";
		&printGrid();
		print "\nline " . ($i + 1) .
		    ": can't place ".&sprintfCase($number)."\n\n";
	    }
	    return -1;
	}
	if ($pos >= 0) {
	    if ($debug) {
		printf "%2i %2i : Fill with %s (line analyse)\n",
		$i + 1, $pos + 1, &sprintfCase($number);
	    }
	    $grid[$i][$pos] = $number;
	    $line[$i] = 0;
	    $column[$pos] = 0;
	    $bloc[&getBlocIndice($i, $pos)] = 0;
	    $ret++;
	}
    }

    # Try to place number in all columns
    for (my $j = 0; $j < $size; $j++) {
	if (!$column[$j]) {
	    next;
	}
	my @ok = (1..$size);
	for (my $i = 0; $i < $size; $i++) {
	    $ok[$i] = 0 if (!$line[$i] || $grid[$i][$j]);
	}
	for (my $k = 0; $k < $subsize; $k++) {
	    if (!$bloc[$k * $subsize + int ($j / $subsize)]) {
		for (my $x = 0; $x < $subsize; $x++) {
		    $ok[$k * $subsize + $x] = 0;
		}
	    }
	}
	my $pos = &getPosNumber(@ok);
	if ($pos == -1) {
	    if ($info) {
		print "Grid after Sudoku algorithm:\n";
		&printGrid();
		print "\ncolumn " . ($j + 1)
		    . ": can't place ".&sprintfCase($number)."\n\n";
	    }
	    return -1;
	}
	if ($pos >= 0) {
	    if ($debug) {
		printf "%2i %2i : Fill with %s (column analyse)\n",
		$pos + 1, $j + 1, &sprintfCase($number);
	    }
	    $grid[$pos][$j] = $number;
	    $line[$pos] = 0;
	    $column[$j] = 0;
	    $bloc[&getBlocIndice($pos, $j)] = 0;
	    $ret++;
	}
    }

    # Try to place number in all bloc
    for (my $k = 0; $k < $size; $k++) {
	if (!$bloc[$k]) {
	    next;
	}
	my @ok = (1..$size);
	my $i2 = int ($k / $subsize);
	my $j2 = $k - $i2 * $subsize;
	$i2 *= $subsize;
	$j2 *= $subsize;
	for (my $p = 0; $p < $subsize; $p++) {
	    for (my $q = 0; $q < $subsize; $q++) {
		if (!$line[$i2 + $p]
		    || !$column[$j2 + $q]
		    || $grid[$i2 + $p][$j2 + $q]) {
		    $ok[$subsize * $p + $q] = 0
		    }
	    }
	}

	my $pos = &getPosNumber(@ok);
	if ($pos == -1) {
	    if ($info) {
		print "Grid after Sudoku algorithm:\n";
		&printGrid();
		print "\nbloc n" . ($k + 1)
		    . ": can't place " . &sprintfCase($number) . "\n\n";
	    }
	    return -1;
	}
	if ($pos >= 0) {
	    my $p = int ($pos / $subsize);
	    my $q = $pos - $p * $subsize;
	    if ($debug) {
		printf "%2i %2i : Fill with %s (bloc analyse)\n",
		$i2 + $p + 1, $j2 + $q + 1, &sprintfCase($number);
	    }
	    $grid[$i2 + $p][$j2 + $q] = $number;
	    $line[$i2 + $p] = 0;
	    $column[$j2 + $q] = 0;
	    $ret++;
	}
    }
    return $ret++;
}

# Try to place some numbers before using brut force
# return 0 is the grid has no solution, 1 otherwise
sub optimization {
    my $nb_pass = 0;
    my $ret = 1;
    my @pass = ();
    # Continue as longer as a number has be placed
    while ($ret) {
	$nb_pass++;
	if ($debug) {
	    print "Pass $nb_pass...\n";
	}
	$ret = 0;
	for (my $num = 1; $num <= $size; $num++) {
	    my $placed = &placeNumber($num);
	    if ($placed == -1) {
		return 0;
	    }
	    $ret += $placed;
	}
	push @pass, $ret if ($ret);
    }
    if ($info) {
	print "Sudoku algorithm:\n";
	print "  Number of pass: " . @pass . "\n";
	my $total = 0;
	for (my $k = 0; $k < @pass; $k++) {
	    print "  Number of fill at pass " . ($k + 1) . ": $pass[$k]\n";
	    $total += $pass[$k];
	}
	print "  Total number of fill: $total\n\n";

	if (&getEmptyNumber() != 0) {
	    print "Grid after Sudoku algorithm:\n";
	    &printGrid();
	}
    }
    return 1;
}



### Brut force algorithm

# Return an array of all numbers that can take place in this case
# Look in the line, columns and square
# NB : $i and $j start at indice 0
sub getPossibilies {
    my ($i, $j) = @_;

    # The array of numbers (true or false)
    # numbers in line/columns/square will be removed
    # first indice isn't used
    my @nine = (0..$size);

    # Remove numbers in line/columns
    for (my $k = 0; $k < $size; $k++) {
	$nine [$grid[$i][$k]] = 0;
	$nine [$grid[$k][$j]] = 0;
    }
    
    # Remove numbers in square
    my $i2 = int ($i / $subsize);
    my $j2 = int ($j / $subsize);
    for (my $p = 0; $p < $subsize; $p++) {
	for (my $q = 0; $q < $subsize; $q++) {
	    $nine [$grid[$i2 * $subsize + $p][$j2 * $subsize + $q]] = 0;
	}
    }

    # Return the array of possibilies
    my @ret = ();
    for (my $k = 1; $k <= $size; $k++) {
	if ($nine[$k]) {
	    push @ret, $k;
	}
    }
    return @ret;
}

# Return the position of the next case to see
# e.g: nextCase(1,1)=(1,2) ... nextCase(1,$size)=(2,1)
sub nextCase {
    my ($i, $j) = @_;
    $j++;
    if ($j > $size) {
	$j = 1;
	$i++;
    }
    return ($i, $j);
}   

# Solve the sudoku
# Return 1 if the sudoku is ok, false otherwise
sub solve {
    # Try to fill as much as possible
    if (! &optimization()) {
	return 0;
    }
    # Find the first case to fill
    my $i = 1;
    my $j = 1;
    while ($i <= $size && $grid[$i-1][$j-1] != 0) {
	($i, $j) = &nextCase($i, $j);
    }
    if ($i > $size) {	# The sudoku was already complete
	&addSolution();
	return 1;
    } else {		# Start solving the sudoku
	&solve2($i, $j);
	if ($info) {
	    print "\nBrut force algorithm:\n";
	    my $total = 0;
	    for (my $k = @try_number - 1; $k > 0; $k--) {
		print "  Number of try between " . $k
		    . " number(s): $try_number[$k]\n";
		$total += $try_number[$k] * $k;
	    }
	    $total += $try_number[0];
	    print "  Number of dead end : $try_number[0]\n";
	    print "  Total number of try: $total\n";
	    if ($allsol) {
		print "  Number of solutions: " . scalar(@solutions) . "\n";
	    }
	    print "\n";
	}
	return @solutions >= 1;
    }
}

# A recursive functon which try to solve sudoku from line $i column $j
# return 1 if $allsol is false and a solution has be found, 0 otherwise
sub solve2 {
    my ($i, $j) = @_;

    my @possibilities = getPossibilies($i - 1, $j - 1);
    $try_number[@possibilities]++;

    if ($debug) {
	printf "%2i %2i : possibilities : ", $i, $j;
	for (@possibilities) {
	    print &sprintfCase($_) . " ";
	}
	print "\n";
    }

    if (@possibilities == 0) {
	if ($debug) {
	    printf "Found a dead end...\n";
	}
    }

    for (@possibilities) {
	if ($debug) {
	    printf "%2i %2i : trying $_\n", $i, $j;
	}
	$grid[$i-1][$j-1] = $_;

	# Find the next case to fill
	my ($new_i, $new_j) = nextCase($i, $j);
	while ($new_i <= $size && $grid[$new_i-1][$new_j-1] != 0) {
	    ($new_i, $new_j) = nextCase($new_i, $new_j);
	}

	# Tests if the sudoku is complete
 	if ($new_i > $size) {
	    &addSolution();
	    if ($debug) {
		printf "Found a solution !\n";
	    }
	    # Don't forget to erase the try
	    $grid[$i-1][$j-1] = 0;
	    return !$allsol;
 	}

	# Follow up to next case
	if (&solve2($new_i, $new_j) != 0) {
	    # Don't forget to erase the try
	    $grid[$i-1][$j-1] = 0;
	    return 1;
	}
    }

    # Don't forget to erase the try
    $grid[$i-1][$j-1] = 0;
    return 0;
}



### Main

&TreatOptions();

my $sol = &solve();
if (!&verifySolution()) {
    print "No solution\n";
    exit 0;
}
if ($sol) {
    if ($info) {
	print "Solution" . (@solutions == 1 ? "" : "s") . ":\n";
    }
    &printSolutions();
} else {
    print "No solution\n";
}



### Doc
__END__

{

=head1 NAME

sudoku - solve sudoku grid

=head1 SYNOPSIS

B<sudoku> [I<options>] [I<GRID>]

=head1 DESCRIPTION

Solve sudoku grid. The grid must have format 4x4 9x9 16x16 25x25...
Solving higher grid should take long time.

The algorithms used to solve grid are really basic.

First, for each number it look if the number can be placed in
line/column/bloc. e.g.: if there is no "2" in the first line, it looks
in which case a "2" can be placed. For that, it look in all columns and in
the blocs crossing the first line if there are some "2". If only a case
in the first line can contain a "2", it place it here.
This algorithm is applied as long as it place at least a number.
The max cost is ridiculous (leaser than n^3 where n is the size of the grid).

Secondly, a brut force recursive algorithm is used. It starts a line 1,
column 1 and for each empty case, look what numbers can be placed here.
It places one and go to the next empty case. If no number can be put in
the case, it's a dead end : it up to the previous filled case and try
another number. The cost for this algorithm is really big.


=head1 GRID FORMAT

Two formats can be used : letter ([a-z]) or numbers.
The elements can be attached or separated by "," or "|".
The numbers greater than 9 must be given separed.
The empty case can be : " " "." "_" "0" for number
and " " "." "_" "`" "@" for letters. An empty string can be used with comma.
  e.g.: B<sudoku> 1000 0210 0030 0104
        B<sudoku> "a   " " ba " "  c " "a  d"
        B<sudoku> A,,, ,B,A, ,,C, ,A,,D

NB: the format must be the same for all lines

=head1 OPTIONS

=over 8

=item B<-a>, B<--all>

Search all solutions.
The default is to stop when the first solution is find.
A good sudoku grid should have only one solution.

=item B<-d>, B<--debug>

Display what the algorithm is doing.

=item B<-f>, B<--file> I<FILE>

Read the grid from file.
The format is a line of the grid per line.
For compatibility, if the first line contains the size of the grid,
it is ignored.
  e.g.:
    4
    1000
    0210
    0030
    0104

=item B<-i>, B<--info>

Display somes informations about the resolutions.

=back

=head1 GNU STANDARD OPTIONS

=over 8

=item B<--help>

Print a usage message on standard output and  exit  successfully.

=item B<--version>

Print version information on standard output, then exit successfully.

=item B<-->

Terminate option list.

=back

=head1 OTHERS OPTIONS

=over 8

=item B<-m>, B<--man>

Print the manual page and exit.

=back

=head1 AUTHOR

Stephane Levant E<lt>sun@tuxfamily.orgE<gt>

=cut

}
